@@ -78,6 +78,9 @@ struct assegment_header
 
 /* Hash for aspath.  This is the top level structure of AS path. */
 struct hash *ashash;
+
+/* Stream for SNMP. See aspath_snmp_pathseg */
+static struct stream *snmp_stream;
 
 static inline as_t *
 assegment_data_new (int num)
@@ -209,11 +212,14 @@ assegment_prepend_asns (struct assegment *seg, as_t asnum, int num)
 static struct assegment *
 assegment_append_asns (struct assegment *seg, as_t *asnos, int num)
 {
-  seg->as = XREALLOC (MTYPE_AS_SEG_DATA, seg->as,
+  as_t *newas;
+  
+  newas = XREALLOC (MTYPE_AS_SEG_DATA, seg->as,
 		      ASSEGMENT_DATA_SIZE (seg->length + num));
 
-  if (seg->as)
+  if (newas)
     {
+      seg->as = newas;
       memcpy (seg->as + seg->length, asnos, ASSEGMENT_DATA_SIZE(num));
       seg->length += num;
       return seg;
@@ -582,9 +588,6 @@ aspath_hash_alloc (void *arg)
   /* New aspath strucutre is needed. */
   aspath = aspath_dup (arg);
   
-  /* Make AS path string. */
-  aspath->str = aspath_make_str_count (aspath);
-
   /* Malformed AS path value. */
   if (! aspath->str)
     {
@@ -596,7 +599,7 @@ aspath_hash_alloc (void *arg)
 }
 
 /* parse as-segment byte stream in struct assegment */
-struct assegment *
+static struct assegment *
 assegments_parse (struct stream *s, size_t length)
 {
   struct assegment_header segh;
@@ -670,6 +673,12 @@ aspath_parse (struct stream *s, size_t length)
   
   /* If already same aspath exist then return it. */
   find = hash_get (ashash, &as, aspath_hash_alloc);
+  
+  /* aspath_hash_alloc dupes segments too. that probably could be
+   * optimised out.
+   */
+  assegment_free_all (as.segments);
+  
   if (! find)
     return NULL;
   find->refcnt++;
@@ -763,22 +772,21 @@ u_char *
 aspath_snmp_pathseg (struct aspath *as, size_t *varlen)
 {
 #define SNMP_PATHSEG_MAX 1024
-  static struct stream *s = NULL;
-  
-  if (!s)
-    s = stream_new (SNMP_PATHSEG_MAX);
+
+  if (!snmp_stream)
+    snmp_stream = stream_new (SNMP_PATHSEG_MAX);
   else
-    stream_reset (s);
+    stream_reset (snmp_stream);
   
   if (!as)
     {
       *varlen = 0;
       return NULL;
     }
-  aspath_put (s, as);
+  aspath_put (snmp_stream, as);
   
-  *varlen = stream_get_endp (s);
-  return stream_pnt(s);
+  *varlen = stream_get_endp (snmp_stream);
+  return stream_pnt(snmp_stream);
 }
       
 #define min(A,B) ((A) < (B) ? (A) : (B))
@@ -918,14 +926,14 @@ aspath_firstas_check (struct aspath *aspath, as_t asno)
   return 0;
 }
 
-/* AS path loop check.  If aspath contains asno then return 1. */
+/* AS path loop check.  If aspath contains asno then return >= 1. */
 int
 aspath_loop_check (struct aspath *aspath, as_t asno)
 {
   struct assegment *seg;
   int count = 0;
 
-  if ( (aspath == NULL) || (aspath->segments) )
+  if ( (aspath == NULL) || (aspath->segments == NULL) )
     return 0;
   
   seg = aspath->segments;
@@ -1155,6 +1163,9 @@ aspath_cmp_left_confed (struct aspath *aspath1, struct aspath *aspath2)
   if (! (aspath1 && aspath2) )
     return 0;
   
+  if ( !(aspath1->segments && aspath2->segments) )
+    return 0;
+  
   if ( (aspath1->segments->type != AS_CONFED_SEQUENCE)
       || (aspath2->segments->type != AS_CONFED_SEQUENCE) )
     return 0;
@@ -1298,37 +1309,30 @@ aspath_gettoken (const char *buf, enum as_token *token, u_short *asno)
     {
     case '\0':
       return NULL;
-      break;
     case '{':
       *token = as_token_set_start;
       p++;
       return p;
-      break;
     case '}':
       *token = as_token_set_end;
       p++;
       return p;
-      break;
     case '(':
       *token = as_token_confed_seq_start;
       p++;
       return p;
-      break;
     case ')':
       *token = as_token_confed_seq_end;
       p++;
       return p;
-      break;
     case '[':
       *token = as_token_confed_set_start;
       p++;
       return p;
-      break;
     case ']':
       *token = as_token_confed_set_end;
       p++;
       return p;
-      break;
     }
 
   /* Check actual AS value. */
@@ -1357,9 +1361,9 @@ aspath_gettoken (const char *buf, enum as_token *token, u_short *asno)
 struct aspath *
 aspath_str2aspath (const char *str)
 {
-  enum as_token token;
+  enum as_token token = as_token_unknown;
   u_short as_type;
-  u_short asno;
+  u_short asno = 0;
   struct aspath *aspath;
   int needtype;
 
@@ -1412,7 +1416,6 @@ aspath_str2aspath (const char *str)
 	default:
 	  aspath_free (aspath);
 	  return NULL;
-	  break;
 	}
     }
 
@@ -1479,6 +1482,15 @@ aspath_init (void)
 {
   ashash = hash_create_size (32767, aspath_key_make, aspath_cmp);
 }
+
+void
+aspath_finish (void)
+{
+  hash_free (ashash);
+  
+  if (snmp_stream)
+    stream_free (snmp_stream);
+}
 
 /* return and as path value */
 const char *
@@ -1489,9 +1501,10 @@ aspath_print (struct aspath *as)
 
 /* Printing functions */
 void
-aspath_print_vty (struct vty *vty, struct aspath *as)
+aspath_print_vty (struct vty *vty, const char *format, struct aspath *as)
 {
-  vty_out (vty, "%s", as->str);
+  assert (format);
+  vty_out (vty, format, as->str);
 }
 
 static void
