@@ -100,7 +100,7 @@ o Local extention
 /* Compares the peer specified in the 'match peer' clause with the peer
     received in bgp_info->peer. If it is the same, or if the peer structure
     received is a peer_group containing it, returns RMAP_MATCH. */
-route_map_result_t
+static route_map_result_t
 route_match_peer (void *rule, struct prefix *prefix, route_map_object_t type,
       void *object)
 {
@@ -124,19 +124,19 @@ route_match_peer (void *rule, struct prefix *prefix, route_map_object_t type,
       su2 = sockunion_str2su ("0.0.0.0");
       if ( sockunion_same (su, su2) )
         {
+          int ret;
           if ( CHECK_FLAG (peer->rmap_type, PEER_RMAP_TYPE_NETWORK) ||
                CHECK_FLAG (peer->rmap_type, PEER_RMAP_TYPE_REDISTRIBUTE) ||
                CHECK_FLAG (peer->rmap_type, PEER_RMAP_TYPE_DEFAULT))
-            {
-              XFREE (MTYPE_SOCKUNION, su2);
-
-              return RMAP_MATCH;
-            }
+            ret = RMAP_MATCH;
           else
-            return RMAP_NOMATCH;
+            ret = RMAP_NOMATCH;
+          
+          sockunion_free (su2);
+          return ret;
         }
-      XFREE (MTYPE_SOCKUNION, su2);
-
+      sockunion_free (su2);
+      
       if (! CHECK_FLAG (peer->sflags, PEER_STATUS_GROUP))
         {
           if (sockunion_same (su, &peer->su))
@@ -159,7 +159,7 @@ route_match_peer (void *rule, struct prefix *prefix, route_map_object_t type,
   return RMAP_NOMATCH;
 }
 
-void *
+static void *
 route_match_peer_compile (const char *arg)
 {
   union sockunion *su;
@@ -177,7 +177,7 @@ route_match_peer_compile (const char *arg)
 }
 
 /* Free route map's compiled `ip address' value. */
-void
+static void
 route_match_peer_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -196,7 +196,7 @@ struct route_map_rule_cmd route_match_peer_cmd =
 
 /* Match function should return 1 if match is success else return
    zero. */
-route_map_result_t
+static route_map_result_t
 route_match_ip_address (void *rule, struct prefix *prefix, 
 			route_map_object_t type, void *object)
 {
@@ -217,14 +217,14 @@ route_match_ip_address (void *rule, struct prefix *prefix,
 
 /* Route map `ip address' match statement.  `arg' should be
    access-list name. */
-void *
+static void *
 route_match_ip_address_compile (const char *arg)
 {
   return XSTRDUP (MTYPE_ROUTE_MAP_COMPILED, arg);
 }
 
 /* Free route map's compiled `ip address' value. */
-void
+static void
 route_match_ip_address_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -242,7 +242,7 @@ struct route_map_rule_cmd route_match_ip_address_cmd =
 /* `match ip next-hop IP_ADDRESS' */
 
 /* Match function return 1 if match is success else return zero. */
-route_map_result_t
+static route_map_result_t
 route_match_ip_next_hop (void *rule, struct prefix *prefix, 
 			 route_map_object_t type, void *object)
 {
@@ -269,14 +269,14 @@ route_match_ip_next_hop (void *rule, struct prefix *prefix,
 
 /* Route map `ip next-hop' match statement. `arg' is
    access-list name. */
-void *
+static void *
 route_match_ip_next_hop_compile (const char *arg)
 {
   return XSTRDUP (MTYPE_ROUTE_MAP_COMPILED, arg);
 }
 
 /* Free route map's compiled `ip address' value. */
-void
+static void
 route_match_ip_next_hop_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -294,7 +294,7 @@ struct route_map_rule_cmd route_match_ip_next_hop_cmd =
 /* `match ip route-source ACCESS-LIST' */
 
 /* Match function return 1 if match is success else return zero. */
-route_map_result_t
+static route_map_result_t
 route_match_ip_route_source (void *rule, struct prefix *prefix, 
 			     route_map_object_t type, void *object)
 {
@@ -327,14 +327,14 @@ route_match_ip_route_source (void *rule, struct prefix *prefix,
 
 /* Route map `ip route-source' match statement. `arg' is
    access-list name. */
-void *
+static void *
 route_match_ip_route_source_compile (const char *arg)
 {
   return XSTRDUP (MTYPE_ROUTE_MAP_COMPILED, arg);
 }
 
 /* Free route map's compiled `ip address' value. */
-void
+static void
 route_match_ip_route_source_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -351,7 +351,7 @@ struct route_map_rule_cmd route_match_ip_route_source_cmd =
 
 /* `match ip address prefix-list PREFIX_LIST' */
 
-route_map_result_t
+static route_map_result_t
 route_match_ip_address_prefix_list (void *rule, struct prefix *prefix, 
 				    route_map_object_t type, void *object)
 {
@@ -369,13 +369,13 @@ route_match_ip_address_prefix_list (void *rule, struct prefix *prefix,
   return RMAP_NOMATCH;
 }
 
-void *
+static void *
 route_match_ip_address_prefix_list_compile (const char *arg)
 {
   return XSTRDUP (MTYPE_ROUTE_MAP_COMPILED, arg);
 }
 
-void
+static void
 route_match_ip_address_prefix_list_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -391,7 +391,7 @@ struct route_map_rule_cmd route_match_ip_address_prefix_list_cmd =
 
 /* `match ip next-hop prefix-list PREFIX_LIST' */
 
-route_map_result_t
+static route_map_result_t
 route_match_ip_next_hop_prefix_list (void *rule, struct prefix *prefix,
                                     route_map_object_t type, void *object)
 {
@@ -416,13 +416,13 @@ route_match_ip_next_hop_prefix_list (void *rule, struct prefix *prefix,
   return RMAP_NOMATCH;
 }
 
-void *
+static void *
 route_match_ip_next_hop_prefix_list_compile (const char *arg)
 {
   return XSTRDUP (MTYPE_ROUTE_MAP_COMPILED, arg);
 }
 
-void
+static void
 route_match_ip_next_hop_prefix_list_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -438,7 +438,7 @@ struct route_map_rule_cmd route_match_ip_next_hop_prefix_list_cmd =
 
 /* `match ip route-source prefix-list PREFIX_LIST' */
 
-route_map_result_t
+static route_map_result_t
 route_match_ip_route_source_prefix_list (void *rule, struct prefix *prefix,
 					 route_map_object_t type, void *object)
 {
@@ -469,13 +469,13 @@ route_match_ip_route_source_prefix_list (void *rule, struct prefix *prefix,
   return RMAP_NOMATCH;
 }
 
-void *
+static void *
 route_match_ip_route_source_prefix_list_compile (const char *arg)
 {
   return XSTRDUP (MTYPE_ROUTE_MAP_COMPILED, arg);
 }
 
-void
+static void
 route_match_ip_route_source_prefix_list_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -492,7 +492,7 @@ struct route_map_rule_cmd route_match_ip_route_source_prefix_list_cmd =
 /* `match metric METRIC' */
 
 /* Match function return 1 if match is success else return zero. */
-route_map_result_t
+static route_map_result_t
 route_match_metric (void *rule, struct prefix *prefix, 
 		    route_map_object_t type, void *object)
 {
@@ -513,7 +513,7 @@ route_match_metric (void *rule, struct prefix *prefix,
 }
 
 /* Route map `match metric' match statement. `arg' is MED value */
-void *
+static void *
 route_match_metric_compile (const char *arg)
 {
   u_int32_t *med;
@@ -534,7 +534,7 @@ route_match_metric_compile (const char *arg)
 }
 
 /* Free route map's compiled `match metric' value. */
-void
+static void
 route_match_metric_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -552,7 +552,7 @@ struct route_map_rule_cmd route_match_metric_cmd =
 /* `match as-path ASPATH' */
 
 /* Match function for as-path match.  I assume given object is */
-route_map_result_t
+static route_map_result_t
 route_match_aspath (void *rule, struct prefix *prefix, 
 		    route_map_object_t type, void *object)
 {
@@ -575,14 +575,14 @@ route_match_aspath (void *rule, struct prefix *prefix,
 }
 
 /* Compile function for as-path match. */
-void *
+static void *
 route_match_aspath_compile (const char *arg)
 {
   return XSTRDUP (MTYPE_ROUTE_MAP_COMPILED, arg);
 }
 
 /* Compile function for as-path match. */
-void
+static void
 route_match_aspath_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -596,55 +596,6 @@ struct route_map_rule_cmd route_match_aspath_cmd =
   route_match_aspath_compile,
   route_match_aspath_free
 };
-
-#if ROUTE_MATCH_ASPATH_OLD
-/* `match as-path ASPATH' */
-
-/* Match function for as-path match.  I assume given object is */
-int
-route_match_aspath (void *rule, struct prefix *prefix, void *object)
-{
-  regex_t *regex;
-  struct bgp_info *bgp_info;
-
-  regex = rule;
-  bgp_info = object;
-  
-  /* Perform match. */
-  return bgp_regexec (regex, bgp_info->attr->aspath);
-}
-
-/* Compile function for as-path match. */
-void *
-route_match_aspath_compile (const char *arg)
-{
-  regex_t *regex;
-
-  regex = bgp_regcomp (arg);
-  if (! regex)
-    return NULL;
-
-  return regex;
-}
-
-/* Compile function for as-path match. */
-void
-route_match_aspath_free (void *rule)
-{
-  regex_t *regex = rule;
-
-  bgp_regex_free (regex);
-}
-
-/* Route map commands for aspath matching. */
-struct route_map_rule_cmd route_match_aspath_cmd = 
-{
-  "as-path",
-  route_match_aspath,
-  route_match_aspath_compile,
-  route_match_aspath_free
-};
-#endif /* ROUTE_MATCH_ASPATH_OLD */
 
 /* `match community COMMUNIY' */
 struct rmap_community
@@ -654,7 +605,7 @@ struct rmap_community
 };
 
 /* Match function for community match. */
-route_map_result_t
+static route_map_result_t
 route_match_community (void *rule, struct prefix *prefix, 
 		       route_map_object_t type, void *object)
 {
@@ -686,7 +637,7 @@ route_match_community (void *rule, struct prefix *prefix,
 }
 
 /* Compile function for community match. */
-void *
+static void *
 route_match_community_compile (const char *arg)
 {
   struct rmap_community *rcom;
@@ -712,7 +663,7 @@ route_match_community_compile (const char *arg)
 }
 
 /* Compile function for community match. */
-void
+static void
 route_match_community_free (void *rule)
 {
   struct rmap_community *rcom = rule;
@@ -731,7 +682,7 @@ struct route_map_rule_cmd route_match_community_cmd =
 };
 
 /* Match function for extcommunity match. */
-route_map_result_t
+static route_map_result_t
 route_match_ecommunity (void *rule, struct prefix *prefix, 
 			route_map_object_t type, void *object)
 {
@@ -754,14 +705,14 @@ route_match_ecommunity (void *rule, struct prefix *prefix,
 }
 
 /* Compile function for extcommunity match. */
-void *
+static void *
 route_match_ecommunity_compile (const char *arg)
 {
   return XSTRDUP (MTYPE_ROUTE_MAP_COMPILED, arg);
 }
 
 /* Compile function for extcommunity match. */
-void
+static void
 route_match_ecommunity_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -780,7 +731,7 @@ struct route_map_rule_cmd route_match_ecommunity_cmd =
    and `address-family vpnv4'.  */
 
 /* `match origin' */
-route_map_result_t
+static route_map_result_t
 route_match_origin (void *rule, struct prefix *prefix, 
 		    route_map_object_t type, void *object)
 {
@@ -799,7 +750,7 @@ route_match_origin (void *rule, struct prefix *prefix,
   return RMAP_NOMATCH;
 }
 
-void *
+static void *
 route_match_origin_compile (const char *arg)
 {
   u_char *origin;
@@ -817,7 +768,7 @@ route_match_origin_compile (const char *arg)
 }
 
 /* Free route map's compiled `ip address' value. */
-void
+static void
 route_match_origin_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -840,7 +791,7 @@ struct rmap_ip_nexthop_set
   int peer_address;
 };
 
-route_map_result_t
+static route_map_result_t
 route_set_ip_nexthop (void *rule, struct prefix *prefix,
 		      route_map_object_t type, void *object)
 {
@@ -887,7 +838,7 @@ route_set_ip_nexthop (void *rule, struct prefix *prefix,
 
 /* Route map `ip nexthop' compile function.  Given string is converted
    to struct in_addr structure. */
-void *
+static void *
 route_set_ip_nexthop_compile (const char *arg)
 {
   struct rmap_ip_nexthop_set *rins;
@@ -919,7 +870,7 @@ route_set_ip_nexthop_compile (const char *arg)
 }
 
 /* Free route map's compiled `ip nexthop' value. */
-void
+static void
 route_set_ip_nexthop_free (void *rule)
 {
   struct rmap_ip_nexthop_set *rins = rule;
@@ -942,7 +893,7 @@ struct route_map_rule_cmd route_set_ip_nexthop_cmd =
 /* `set local-preference LOCAL_PREF' */
 
 /* Set local preference. */
-route_map_result_t
+static route_map_result_t
 route_set_local_pref (void *rule, struct prefix *prefix,
 		      route_map_object_t type, void *object)
 {
@@ -964,7 +915,7 @@ route_set_local_pref (void *rule, struct prefix *prefix,
 }
 
 /* set local preference compilation. */
-void *
+static void *
 route_set_local_pref_compile (const char *arg)
 {
   unsigned long tmp;
@@ -990,7 +941,7 @@ route_set_local_pref_compile (const char *arg)
 }
 
 /* Free route map's local preference value. */
-void
+static void
 route_set_local_pref_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -1008,7 +959,7 @@ struct route_map_rule_cmd route_set_local_pref_cmd =
 /* `set weight WEIGHT' */
 
 /* Set weight. */
-route_map_result_t
+static route_map_result_t
 route_set_weight (void *rule, struct prefix *prefix, route_map_object_t type,
 		  void *object)
 {
@@ -1029,7 +980,7 @@ route_set_weight (void *rule, struct prefix *prefix, route_map_object_t type,
 }
 
 /* set local preference compilation. */
-void *
+static void *
 route_set_weight_compile (const char *arg)
 {
   unsigned long tmp;
@@ -1056,7 +1007,7 @@ route_set_weight_compile (const char *arg)
 }
 
 /* Free route map's local preference value. */
-void
+static void
 route_set_weight_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -1074,7 +1025,7 @@ struct route_map_rule_cmd route_set_weight_cmd =
 /* `set metric METRIC' */
 
 /* Set metric to attribute. */
-route_map_result_t
+static route_map_result_t
 route_set_metric (void *rule, struct prefix *prefix, 
 		  route_map_object_t type, void *object)
 {
@@ -1121,18 +1072,20 @@ route_set_metric (void *rule, struct prefix *prefix,
 }
 
 /* set metric compilation. */
-void *
+static void *
 route_set_metric_compile (const char *arg)
 {
   u_int32_t metric;
+  unsigned long larg;
   char *endptr = NULL;
 
   if (all_digit (arg))
     {
       /* set metric value check*/
-      metric = strtoul (arg, &endptr, 10);
-      if (*endptr != '\0' || metric == ULONG_MAX)
+      larg = strtoul (arg, &endptr, 10);
+      if (*endptr != '\0' || larg == ULONG_MAX || larg > UINT32_MAX)
         return NULL;
+      metric = larg;
     }
   else
     {
@@ -1142,16 +1095,17 @@ route_set_metric_compile (const char *arg)
 	   || (! all_digit (arg+1)))
 	return NULL;
 
-      metric = strtoul (arg+1, &endptr, 10);
-      if (*endptr != '\0' || metric == ULONG_MAX)
+      larg = strtoul (arg+1, &endptr, 10);
+      if (*endptr != '\0' || larg == ULONG_MAX || larg > UINT32_MAX)
 	return NULL;
+      metric = larg;
     }
 
   return XSTRDUP (MTYPE_ROUTE_MAP_COMPILED, arg);
 }
 
 /* Free route map's compiled `set metric' value. */
-void
+static void
 route_set_metric_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -1169,7 +1123,7 @@ struct route_map_rule_cmd route_set_metric_cmd =
 /* `set as-path prepend ASPATH' */
 
 /* For AS path prepend mechanism. */
-route_map_result_t
+static route_map_result_t
 route_set_aspath_prepend (void *rule, struct prefix *prefix, route_map_object_t type, void *object)
 {
   struct aspath *aspath;
@@ -1194,7 +1148,7 @@ route_set_aspath_prepend (void *rule, struct prefix *prefix, route_map_object_t
 }
 
 /* Compile function for as-path prepend. */
-void *
+static void *
 route_set_aspath_prepend_compile (const char *arg)
 {
   struct aspath *aspath;
@@ -1206,7 +1160,7 @@ route_set_aspath_prepend_compile (const char *arg)
 }
 
 /* Compile function for as-path prepend. */
-void
+static void
 route_set_aspath_prepend_free (void *rule)
 {
   struct aspath *aspath = rule;
@@ -1231,7 +1185,7 @@ struct rmap_com_set
 };
 
 /* For community set mechanism. */
-route_map_result_t
+static route_map_result_t
 route_set_community (void *rule, struct prefix *prefix,
 		     route_map_object_t type, void *object)
 {
@@ -1241,7 +1195,7 @@ route_set_community (void *rule, struct prefix *prefix,
   struct community *new = NULL;
   struct community *old;
   struct community *merge;
-
+  
   if (type == RMAP_BGP)
     {
       rcs = rule;
@@ -1261,13 +1215,22 @@ route_set_community (void *rule, struct prefix *prefix,
       if (rcs->additive && old)
 	{
 	  merge = community_merge (community_dup (old), rcs->com);
+	  
+	  /* HACK: if the old community is not intern'd, 
+           * we should free it here, or all reference to it may be lost.
+           * Really need to cleanup attribute caching sometime.
+           */
+	  if (old->refcnt == 0)
+	    community_free (old);
 	  new = community_uniq_sort (merge);
 	  community_free (merge);
 	}
       else
 	new = community_dup (rcs->com);
-
+      
+      /* will be interned by caller if required */
       attr->community = new;
+
       attr->flag |= ATTR_FLAG_BIT (BGP_ATTR_COMMUNITIES);
     }
 
@@ -1275,7 +1238,7 @@ route_set_community (void *rule, struct prefix *prefix,
 }
 
 /* Compile function for set community. */
-void *
+static void *
 route_set_community_compile (const char *arg)
 {
   struct rmap_com_set *rcs;
@@ -1317,7 +1280,7 @@ route_set_community_compile (const char *arg)
 }
 
 /* Free function for set community. */
-void
+static void
 route_set_community_free (void *rule)
 {
   struct rmap_com_set *rcs = rule;
@@ -1339,7 +1302,7 @@ struct route_map_rule_cmd route_set_community_cmd =
 /* `set comm-list (<1-99>|<100-500>|WORD) delete' */
 
 /* For community set mechanism. */
-route_map_result_t
+static route_map_result_t
 route_set_community_delete (void *rule, struct prefix *prefix,
 			    route_map_object_t type, void *object)
 {
@@ -1382,7 +1345,7 @@ route_set_community_delete (void *rule, struct prefix *prefix,
 }
 
 /* Compile function for set community. */
-void *
+static void *
 route_set_community_delete_compile (const char *arg)
 {
   char *p;
@@ -1403,7 +1366,7 @@ route_set_community_delete_compile (const char *arg)
 }
 
 /* Free function for set community. */
-void
+static void
 route_set_community_delete_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -1421,7 +1384,7 @@ struct route_map_rule_cmd route_set_community_delete_cmd =
 /* `set extcommunity rt COMMUNITY' */
 
 /* For community set mechanism. */
-route_map_result_t
+static route_map_result_t
 route_set_ecommunity_rt (void *rule, struct prefix *prefix, 
 			 route_map_object_t type, void *object)
 {
@@ -1448,13 +1411,16 @@ route_set_ecommunity_rt (void *rule, struct prefix *prefix,
 
       bgp_info->attr->ecommunity = new_ecom;
 
+      if (old_ecom)
+	ecommunity_free (old_ecom);
+
       bgp_info->attr->flag |= ATTR_FLAG_BIT (BGP_ATTR_EXT_COMMUNITIES);
     }
   return RMAP_OKAY;
 }
 
 /* Compile function for set community. */
-void *
+static void *
 route_set_ecommunity_rt_compile (const char *arg)
 {
   struct ecommunity *ecom;
@@ -1466,7 +1432,7 @@ route_set_ecommunity_rt_compile (const char *arg)
 }
 
 /* Free function for set community. */
-void
+static void
 route_set_ecommunity_rt_free (void *rule)
 {
   struct ecommunity *ecom = rule;
@@ -1485,7 +1451,7 @@ struct route_map_rule_cmd route_set_ecommunity_rt_cmd =
 /* `set extcommunity soo COMMUNITY' */
 
 /* For community set mechanism. */
-route_map_result_t
+static route_map_result_t
 route_set_ecommunity_soo (void *rule, struct prefix *prefix, 
 			 route_map_object_t type, void *object)
 {
@@ -1507,7 +1473,7 @@ route_set_ecommunity_soo (void *rule, struct prefix *prefix,
 }
 
 /* Compile function for set community. */
-void *
+static void *
 route_set_ecommunity_soo_compile (const char *arg)
 {
   struct ecommunity *ecom;
@@ -1520,7 +1486,7 @@ route_set_ecommunity_soo_compile (const char *arg)
 }
 
 /* Free function for set community. */
-void
+static void
 route_set_ecommunity_soo_free (void *rule)
 {
   struct ecommunity *ecom = rule;
@@ -1539,7 +1505,7 @@ struct route_map_rule_cmd route_set_ecommunity_soo_cmd =
 /* `set origin ORIGIN' */
 
 /* For origin set. */
-route_map_result_t
+static route_map_result_t
 route_set_origin (void *rule, struct prefix *prefix, route_map_object_t type, void *object)
 {
   u_char *origin;
@@ -1557,7 +1523,7 @@ route_set_origin (void *rule, struct prefix *prefix, route_map_object_t type, vo
 }
 
 /* Compile function for origin set. */
-void *
+static void *
 route_set_origin_compile (const char *arg)
 {
   u_char *origin;
@@ -1575,7 +1541,7 @@ route_set_origin_compile (const char *arg)
 }
 
 /* Compile function for origin set. */
-void
+static void
 route_set_origin_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -1593,7 +1559,7 @@ struct route_map_rule_cmd route_set_origin_cmd =
 /* `set atomic-aggregate' */
 
 /* For atomic aggregate set. */
-route_map_result_t
+static route_map_result_t
 route_set_atomic_aggregate (void *rule, struct prefix *prefix,
 			    route_map_object_t type, void *object)
 {
@@ -1609,14 +1575,14 @@ route_set_atomic_aggregate (void *rule, struct prefix *prefix,
 }
 
 /* Compile function for atomic aggregate. */
-void *
+static void *
 route_set_atomic_aggregate_compile (const char *arg)
 {
   return (void *)1;
 }
 
 /* Compile function for atomic aggregate. */
-void
+static void
 route_set_atomic_aggregate_free (void *rule)
 {
   return;
@@ -1638,7 +1604,7 @@ struct aggregator
   struct in_addr address;
 };
 
-route_map_result_t
+static route_map_result_t
 route_set_aggregator_as (void *rule, struct prefix *prefix, 
 			 route_map_object_t type, void *object)
 {
@@ -1658,7 +1624,7 @@ route_set_aggregator_as (void *rule, struct prefix *prefix,
   return RMAP_OKAY;
 }
 
-void *
+static void *
 route_set_aggregator_as_compile (const char *arg)
 {
   struct aggregator *aggregator;
@@ -1676,7 +1642,7 @@ route_set_aggregator_as_compile (const char *arg)
   return aggregator;
 }
 
-void
+static void
 route_set_aggregator_as_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -1693,7 +1659,7 @@ struct route_map_rule_cmd route_set_aggregator_as_cmd =
 #ifdef HAVE_IPV6
 /* `match ipv6 address IP_ACCESS_LIST' */
 
-route_map_result_t
+static route_map_result_t
 route_match_ipv6_address (void *rule, struct prefix *prefix, 
 			  route_map_object_t type, void *object)
 {
@@ -1711,13 +1677,13 @@ route_match_ipv6_address (void *rule, struct prefix *prefix,
   return RMAP_NOMATCH;
 }
 
-void *
+static void *
 route_match_ipv6_address_compile (const char *arg)
 {
   return XSTRDUP (MTYPE_ROUTE_MAP_COMPILED, arg);
 }
 
-void
+static void
 route_match_ipv6_address_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -1734,7 +1700,7 @@ struct route_map_rule_cmd route_match_ipv6_address_cmd =
 
 /* `match ipv6 next-hop IP_ADDRESS' */
 
-route_map_result_t
+static route_map_result_t
 route_match_ipv6_next_hop (void *rule, struct prefix *prefix, 
 			   route_map_object_t type, void *object)
 {
@@ -1759,7 +1725,7 @@ route_match_ipv6_next_hop (void *rule, struct prefix *prefix,
   return RMAP_NOMATCH;
 }
 
-void *
+static void *
 route_match_ipv6_next_hop_compile (const char *arg)
 {
   struct in6_addr *address;
@@ -1777,7 +1743,7 @@ route_match_ipv6_next_hop_compile (const char *arg)
   return address;
 }
 
-void
+static void
 route_match_ipv6_next_hop_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -1793,7 +1759,7 @@ struct route_map_rule_cmd route_match_ipv6_next_hop_cmd =
 
 /* `match ipv6 address prefix-list PREFIX_LIST' */
 
-route_map_result_t
+static route_map_result_t
 route_match_ipv6_address_prefix_list (void *rule, struct prefix *prefix, 
 			      route_map_object_t type, void *object)
 {
@@ -1811,13 +1777,13 @@ route_match_ipv6_address_prefix_list (void *rule, struct prefix *prefix,
   return RMAP_NOMATCH;
 }
 
-void *
+static void *
 route_match_ipv6_address_prefix_list_compile (const char *arg)
 {
   return XSTRDUP (MTYPE_ROUTE_MAP_COMPILED, arg);
 }
 
-void
+static void
 route_match_ipv6_address_prefix_list_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -1834,7 +1800,7 @@ struct route_map_rule_cmd route_match_ipv6_address_prefix_list_cmd =
 /* `set ipv6 nexthop global IP_ADDRESS' */
 
 /* Set nexthop to object.  ojbect must be pointer to struct attr. */
-route_map_result_t
+static route_map_result_t
 route_set_ipv6_nexthop_global (void *rule, struct prefix *prefix, 
 			       route_map_object_t type, void *object)
 {
@@ -1860,7 +1826,7 @@ route_set_ipv6_nexthop_global (void *rule, struct prefix *prefix,
 
 /* Route map `ip next-hop' compile function.  Given string is converted
    to struct in_addr structure. */
-void *
+static void *
 route_set_ipv6_nexthop_global_compile (const char *arg)
 {
   int ret;
@@ -1880,7 +1846,7 @@ route_set_ipv6_nexthop_global_compile (const char *arg)
 }
 
 /* Free route map's compiled `ip next-hop' value. */
-void
+static void
 route_set_ipv6_nexthop_global_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -1898,7 +1864,7 @@ struct route_map_rule_cmd route_set_ipv6_nexthop_global_cmd =
 /* `set ipv6 nexthop local IP_ADDRESS' */
 
 /* Set nexthop to object.  ojbect must be pointer to struct attr. */
-route_map_result_t
+static route_map_result_t
 route_set_ipv6_nexthop_local (void *rule, struct prefix *prefix, 
 			      route_map_object_t type, void *object)
 {
@@ -1924,7 +1890,7 @@ route_set_ipv6_nexthop_local (void *rule, struct prefix *prefix,
 
 /* Route map `ip nexthop' compile function.  Given string is converted
    to struct in_addr structure. */
-void *
+static void *
 route_set_ipv6_nexthop_local_compile (const char *arg)
 {
   int ret;
@@ -1944,7 +1910,7 @@ route_set_ipv6_nexthop_local_compile (const char *arg)
 }
 
 /* Free route map's compiled `ip nexthop' value. */
-void
+static void
 route_set_ipv6_nexthop_local_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -1962,7 +1928,7 @@ struct route_map_rule_cmd route_set_ipv6_nexthop_local_cmd =
 
 /* `set vpnv4 nexthop A.B.C.D' */
 
-route_map_result_t
+static route_map_result_t
 route_set_vpnv4_nexthop (void *rule, struct prefix *prefix, 
 			 route_map_object_t type, void *object)
 {
@@ -1982,7 +1948,7 @@ route_set_vpnv4_nexthop (void *rule, struct prefix *prefix,
   return RMAP_OKAY;
 }
 
-void *
+static void *
 route_set_vpnv4_nexthop_compile (const char *arg)
 {
   int ret;
@@ -2001,7 +1967,7 @@ route_set_vpnv4_nexthop_compile (const char *arg)
   return address;
 }
 
-void
+static void
 route_set_vpnv4_nexthop_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -2019,7 +1985,7 @@ struct route_map_rule_cmd route_set_vpnv4_nexthop_cmd =
 /* `set originator-id' */
 
 /* For origin set. */
-route_map_result_t
+static route_map_result_t
 route_set_originator_id (void *rule, struct prefix *prefix, route_map_object_t type, void *object)
 {
   struct in_addr *address;
@@ -2038,7 +2004,7 @@ route_set_originator_id (void *rule, struct prefix *prefix, route_map_object_t t
 }
 
 /* Compile function for originator-id set. */
-void *
+static void *
 route_set_originator_id_compile (const char *arg)
 {
   int ret;
@@ -2058,7 +2024,7 @@ route_set_originator_id_compile (const char *arg)
 }
 
 /* Compile function for originator_id set. */
-void
+static void
 route_set_originator_id_free (void *rule)
 {
   XFREE (MTYPE_ROUTE_MAP_COMPILED, rule);
@@ -2074,7 +2040,7 @@ struct route_map_rule_cmd route_set_originator_id_cmd =
 };
 
 /* Add bgp route map rule. */
-int
+static int
 bgp_route_match_add (struct vty *vty, struct route_map_index *index,
 		     const char *command, const char *arg)
 {
@@ -2088,18 +2054,16 @@ bgp_route_match_add (struct vty *vty, struct route_map_index *index,
 	case RMAP_RULE_MISSING:
 	  vty_out (vty, "%% Can't find rule.%s", VTY_NEWLINE);
 	  return CMD_WARNING;
-	  break;
 	case RMAP_COMPILE_ERROR:
 	  vty_out (vty, "%% Argument is malformed.%s", VTY_NEWLINE);
 	  return CMD_WARNING;
-	  break;
 	}
     }
   return CMD_SUCCESS;
 }
 
 /* Delete bgp route map rule. */
-int
+static int
 bgp_route_match_delete (struct vty *vty, struct route_map_index *index,
 			const char *command, const char *arg)
 {
@@ -2113,18 +2077,16 @@ bgp_route_match_delete (struct vty *vty, struct route_map_index *index,
 	case RMAP_RULE_MISSING:
 	  vty_out (vty, "%% Can't find rule.%s", VTY_NEWLINE);
 	  return CMD_WARNING;
-	  break;
 	case RMAP_COMPILE_ERROR:
 	  vty_out (vty, "%% Argument is malformed.%s", VTY_NEWLINE);
 	  return CMD_WARNING;
-	  break;
 	}
     }
   return CMD_SUCCESS;
 }
 
 /* Add bgp route map rule. */
-int
+static int
 bgp_route_set_add (struct vty *vty, struct route_map_index *index,
 		   const char *command, const char *arg)
 {
@@ -2138,18 +2100,16 @@ bgp_route_set_add (struct vty *vty, struct route_map_index *index,
 	case RMAP_RULE_MISSING:
 	  vty_out (vty, "%% Can't find rule.%s", VTY_NEWLINE);
 	  return CMD_WARNING;
-	  break;
 	case RMAP_COMPILE_ERROR:
 	  vty_out (vty, "%% Argument is malformed.%s", VTY_NEWLINE);
 	  return CMD_WARNING;
-	  break;
 	}
     }
   return CMD_SUCCESS;
 }
 
 /* Delete bgp route map rule. */
-int
+static int
 bgp_route_set_delete (struct vty *vty, struct route_map_index *index,
 		      const char *command, const char *arg)
 {
@@ -2163,18 +2123,16 @@ bgp_route_set_delete (struct vty *vty, struct route_map_index *index,
 	case RMAP_RULE_MISSING:
 	  vty_out (vty, "%% Can't find rule.%s", VTY_NEWLINE);
 	  return CMD_WARNING;
-	  break;
 	case RMAP_COMPILE_ERROR:
 	  vty_out (vty, "%% Argument is malformed.%s", VTY_NEWLINE);
 	  return CMD_WARNING;
-	  break;
 	}
     }
   return CMD_SUCCESS;
 }
 
 /* Hook function for updating route_map assignment. */
-void
+static void
 bgp_route_map_update (const char *unused)
 {
   int i;
@@ -2796,7 +2754,7 @@ DEFUN (set_ip_nexthop_peer,
   return bgp_route_set_add (vty, vty->index, "ip next-hop", "peer-address");
 }
 
-DEFUN (no_set_ip_nexthop_peer,
+DEFUN_DEPRECATED (no_set_ip_nexthop_peer,
        no_set_ip_nexthop_peer_cmd,
        "no set ip next-hop peer-address",
        NO_STR
@@ -2846,7 +2804,7 @@ ALIAS (set_metric,
        "set metric <+/-metric>",
        SET_STR
        "Metric value for destination routing protocol\n"
-       "Add or subtract BGP metric\n")
+       "Add or subtract metric\n")
 
 DEFUN (no_set_metric,
        no_set_metric_cmd,
@@ -3288,7 +3246,7 @@ DEFUN (set_aggregator_as,
   struct in_addr address;
   char *argstr;
 
-  VTY_GET_INTEGER_RANGE ("AS Path", as, argv[0], 1, BGP_AS_MAX)
+  VTY_GET_INTEGER_RANGE ("AS Path", as, argv[0], 1, BGP_AS_MAX);
   
   ret = inet_aton (argv[1], &address);
   if (ret == 0)
@@ -3325,7 +3283,7 @@ DEFUN (no_set_aggregator_as,
   if (argv == 0)
     return bgp_route_set_delete (vty, vty->index, "aggregator as", NULL);
   
-  VTY_GET_INTEGER_RANGE ("AS Path", as, argv[0], 1, BGP_AS_MAX)  
+  VTY_GET_INTEGER_RANGE ("AS Path", as, argv[0], 1, BGP_AS_MAX);
 
   ret = inet_aton (argv[1], &address);
   if (ret == 0)
@@ -3572,7 +3530,7 @@ ALIAS (no_set_originator_id,
 
 /* Initialization of route map. */
 void
-bgp_route_map_init ()
+bgp_route_map_init (void)
 {
   route_map_init ();
   route_map_init_vty ();
