@@ -20,7 +20,6 @@
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#include <stdio.h>
 #include <zebra.h>
 
 #include "getopt.h"
@@ -52,7 +51,7 @@
 
 /* isisd privileges */
 zebra_capabilities_t _caps_p[] = {
-  ZCAP_RAW,
+  ZCAP_NET_RAW,
   ZCAP_BIND
 };
 
@@ -79,6 +78,7 @@ struct option longopts[] = {
   {"vty_addr", required_argument, NULL, 'A'},
   {"vty_port", required_argument, NULL, 'P'},
   {"user", required_argument, NULL, 'u'},
+  {"group", required_argument, NULL, 'g'},
   {"version", no_argument, NULL, 'v'},
   {"help", no_argument, NULL, 'h'},
   {0}
@@ -121,7 +121,8 @@ Daemon which manages IS-IS routing\n\n\
 -i, --pid_file     Set process identifier file name\n\
 -A, --vty_addr     Set vty's bind address\n\
 -P, --vty_port     Set vty's port number\n\
--u, --user         User and group to run as\n\
+-u, --user         User to run as\n\
+-g, --group        Group to run as\n\
 -v, --version      Print program version\n\
 -h, --help         Display this help and exit\n\
 \n\
@@ -135,13 +136,13 @@ Report bugs to http://bugzilla.quagga.net\n", progname);
 void
 reload ()
 {
-  zlog_info ("Reload");
+  zlog_debug ("Reload");
   /* FIXME: Clean up func call here */
-  vty_finish ();
+  vty_reset ();
   execve (_progpath, _argv, _envp);
 }
 
-void
+static void
 terminate (int i)
 {
   exit (i);
@@ -154,7 +155,7 @@ terminate (int i)
 void
 sighup (void)
 {
-  zlog_info ("SIGHUP received");
+  zlog_debug ("SIGHUP received");
   reload ();
 
   return;
@@ -163,23 +164,21 @@ sighup (void)
 void
 sigint (void)
 {
-  zlog_info ("SIGINT received");
+  zlog_notice ("Terminating on signal SIGINT");
   terminate (0);
-
-  return;
 }
 
 void
 sigterm (void)
 {
-  zlog_info ("SIGTERM received");
+  zlog_notice ("Terminating on signal SIGTERM");
   terminate (0);
 }
 
 void
 sigusr1 (void)
 {
-  zlog_info ("SIGUSR1 received");
+  zlog_debug ("SIGUSR1 received");
   zlog_rotate (NULL);
 }
 
@@ -218,7 +217,7 @@ main (int argc, char **argv, char **envp)
   /* Get the programname without the preceding path. */
   progname = ((p = strrchr (argv[0], '/')) ? ++p : argv[0]);
 
-  zlog_default = openzlog (progname, ZLOG_NOLOG, ZLOG_ISIS,
+  zlog_default = openzlog (progname, ZLOG_ISIS,
 			   LOG_CONS | LOG_NDELAY | LOG_PID, LOG_DAEMON);
 
   /* for reload */
@@ -234,7 +233,7 @@ main (int argc, char **argv, char **envp)
   /* Command line argument treatment. */
   while (1)
     {
-      opt = getopt_long (argc, argv, "df:i:hA:p:P:u:v", longopts, 0);
+      opt = getopt_long (argc, argv, "df:i:hA:p:P:u:g:v", longopts, 0);
 
       if (opt == EOF)
 	break;
@@ -267,8 +266,10 @@ main (int argc, char **argv, char **envp)
 	  vty_port = (vty_port ? vty_port : ISISD_VTY_PORT);
 	  break;
 	case 'u':
-	  isisd_privs.user = isisd_privs.group = optarg;
+	  isisd_privs.user = optarg;
 	  break;
+	case 'g':
+	  isisd_privs.group = optarg;
 	  break;
 	case 'v':
 	  printf ("ISISd version %s\n", ISISD_VERSION);
@@ -322,14 +323,8 @@ main (int argc, char **argv, char **envp)
   vty_serv_sock (vty_addr, vty_port, ISIS_VTYSH_PATH);
 
   /* Print banner. */
-#if defined(ZEBRA_VERSION)
-  zlog_info ("ISISd %s starting: vty@%d", ZEBRA_VERSION, vty_port);
-#elif defined(QUAGGA_VERSION)
-  zlog_info ("Quagga-ISISd %s starting: vty@%d", QUAGGA_VERSION, vty_port);
-#endif
-#ifdef HAVE_IPV6
-  zlog_info ("IPv6 enabled");
-#endif
+  zlog_notice ("Quagga-ISISd %s starting: vty@%d", QUAGGA_VERSION, vty_port);
+
   /* Start finite state machine. */
   while (thread_fetch (master, &thread))
     thread_call (&thread);
