@@ -22,8 +22,6 @@
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
 #include <zebra.h>
 
 #include "thread.h"
