@@ -191,13 +191,13 @@ if_lookup_by_index (unsigned int index)
   return NULL;
 }
 
-char *
+const char *
 ifindex2ifname (unsigned int index)
 {
   struct interface *ifp;
 
   return ((ifp = if_lookup_by_index(index)) != NULL) ?
-  	 ifp->name : (char *)"unknown";
+  	 ifp->name : "unknown";
 }
 
 unsigned int
@@ -405,7 +405,7 @@ if_flag_dump (unsigned long flag)
       strlcat (logbuf, STR, BUFSIZ); \
     }
 
-  strlcpy (logbuf, "  <", BUFSIZ);
+  strlcpy (logbuf, "<", BUFSIZ);
   IFF_OUT_LOG (IFF_UP, "UP");
   IFF_OUT_LOG (IFF_BROADCAST, "BROADCAST");
   IFF_OUT_LOG (IFF_DEBUG, "DEBUG");
@@ -431,10 +431,11 @@ if_flag_dump (unsigned long flag)
   strlcat (logbuf, ">", BUFSIZ);
 
   return logbuf;
+#undef IFF_OUT_LOG
 }
 
 /* For debugging */
-void
+static void
 if_dump (struct interface *ifp)
 {
   struct listnode *node;
@@ -495,7 +496,7 @@ DEFUN (no_interface_desc,
 
   ifp = vty->index;
   if (ifp->desc)
-    XFREE (0, ifp->desc);
+    XFREE (MTYPE_TMP, ifp->desc);
   ifp->desc = NULL;
 
   return CMD_SUCCESS;
@@ -587,7 +588,7 @@ DEFUN (show_address,
 
 /* Allocate connected structure. */
 struct connected *
-connected_new ()
+connected_new (void)
 {
   struct connected *new = XMALLOC (MTYPE_CONNECTED, sizeof (struct connected));
   memset (new, 0, sizeof (struct connected));
@@ -605,13 +606,13 @@ connected_free (struct connected *connected)
     prefix_free (connected->destination);
 
   if (connected->label)
-    free (connected->label);
+    XFREE (MTYPE_CONNECTED_LABEL, connected->label);
 
   XFREE (MTYPE_CONNECTED, connected);
 }
 
 /* Print if_addr structure. */
-void
+static void __attribute__ ((unused))
 connected_log (struct connected *connected, char *str)
 {
   struct prefix *p;
@@ -637,7 +638,7 @@ connected_log (struct connected *connected, char *str)
 }
 
 /* If two connected address has same prefix return 1. */
-int
+static int
 connected_same_prefix (struct prefix *p1, struct prefix *p2)
 {
   if (p1->family == p2->family)
@@ -767,13 +768,16 @@ if_indextoname (unsigned int ifindex, char *name)
 }
 #endif
 
+#if 0 /* this route_table of struct connected's is unused
+       * however, it would be good to use a route_table rather than
+       * a list..
+       */
 /* Interface looking up by interface's address. */
-
 /* Interface's IPv4 address reverse lookup table. */
 struct route_table *ifaddr_ipv4_table;
 /* struct route_table *ifaddr_ipv6_table; */
 
-void
+static void
 ifaddr_ipv4_add (struct in_addr *ifaddr, struct interface *ifp)
 {
   struct route_node *rn;
@@ -794,7 +798,7 @@ ifaddr_ipv4_add (struct in_addr *ifaddr, struct interface *ifp)
   rn->info = ifp;
 }
 
-void
+static void
 ifaddr_ipv4_delete (struct in_addr *ifaddr, struct interface *ifp)
 {
   struct route_node *rn;
@@ -817,7 +821,7 @@ ifaddr_ipv4_delete (struct in_addr *ifaddr, struct interface *ifp)
 }
 
 /* Lookup interface by interface's IP address or interface index. */
-struct interface *
+static struct interface *
 ifaddr_ipv4_lookup (struct in_addr *addr, unsigned int ifindex)
 {
   struct prefix_ipv4 p;
@@ -841,13 +845,16 @@ ifaddr_ipv4_lookup (struct in_addr *addr, unsigned int ifindex)
   else
     return if_lookup_by_index(ifindex);
 }
+#endif /* ifaddr_ipv4_table */
 
 /* Initialize interface list. */
 void
-if_init ()
+if_init (void)
 {
   iflist = list_new ();
+#if 0
   ifaddr_ipv4_table = route_table_init ();
+#endif /* ifaddr_ipv4_table */
 
   if (iflist) {
     iflist->cmp = (int (*)(void *, void *))if_cmp_func;
