@@ -1,5 +1,5 @@
 /*
- * $Id: log.c,v 1.20.2.3 2005/01/23 20:45:17 hasso Exp $
+ * $Id: log.c,v 1.28 2006/05/28 08:26:15 paul Exp $
  *
  * Logging of zebra
  * Copyright (C) 1997, 1998, 1999 Kunihiro Ishiguro
@@ -31,6 +31,8 @@
 #include <sys/un.h>
 #endif
 
+static int logfile_fd = -1;	/* Used in signal handler. */
+
 struct zlog *zlog_default = NULL;
 
 const char *zlog_proto_names[] = 
@@ -177,7 +179,7 @@ num_append(char *s, int len, u_long x)
   return str_append(s,len,t);
 }
 
-#if defined(SA_SIGINFO) || defined(HAVE_GLIBC_BACKTRACE)
+#if defined(SA_SIGINFO) || defined(HAVE_STACK_TRACE)
 static char *
 hex_append(char *s, int len, u_long x)
 {
@@ -197,8 +199,6 @@ hex_append(char *s, int len, u_long x)
 }
 #endif
 
-static int syslog_fd = -1;
-
 /* Needs to be enhanced to support Solaris. */
 static int
 syslog_connect(void)
@@ -233,6 +233,7 @@ syslog_connect(void)
 static void
 syslog_sigsafe(int priority, const char *msg, size_t msglen)
 {
+  static int syslog_fd = -1;
   char buf[sizeof("<1234567890>ripngd[1234567890]: ")+msglen+50];
   char *s;
 
@@ -258,6 +259,38 @@ syslog_sigsafe(int priority, const char *msg, size_t msglen)
 #undef LOC
 }
 
+static int
+open_crashlog(void)
+{
+#define CRASHLOG_PREFIX "/var/tmp/quagga."
+#define CRASHLOG_SUFFIX "crashlog"
+  if (zlog_default && zlog_default->ident)
+    {
+      /* Avoid strlen since it is not async-signal-safe. */
+      const char *p;
+      size_t ilen;
+
+      for (p = zlog_default->ident, ilen = 0; *p; p++)
+	ilen++;
+      {
+	char buf[sizeof(CRASHLOG_PREFIX)+ilen+sizeof(CRASHLOG_SUFFIX)+3];
+	char *s = buf;
+#define LOC s,buf+sizeof(buf)-s
+	s = str_append(LOC, CRASHLOG_PREFIX);
+	s = str_append(LOC, zlog_default->ident);
+	s = str_append(LOC, ".");
+	s = str_append(LOC, CRASHLOG_SUFFIX);
+#undef LOC
+	*s = '\0';
+	return open(buf, O_WRONLY|O_CREAT|O_EXCL, LOGFILE_MASK);
+      }
+    }
+  return open(CRASHLOG_PREFIX CRASHLOG_SUFFIX, O_WRONLY|O_CREAT|O_EXCL,
+	      LOGFILE_MASK);
+#undef CRASHLOG_SUFFIX
+#undef CRASHLOG_PREFIX
+}
+
 /* Note: the goal here is to use only async-signal-safe functions. */
 void
 zlog_signal(int signo, const char *action
@@ -301,17 +334,18 @@ zlog_signal(int signo, const char *action
     *s++ = '\n';
 
   /* N.B. implicit priority is most severe */
-#define PRI LOG_ERR
+#define PRI LOG_CRIT
 
-#define DUMP(FP) write(fileno(FP),buf,s-buf);
+#define DUMP(FD) write(FD, buf, s-buf);
+  /* If no file logging configured, try to write to fallback log file. */
+  if ((logfile_fd >= 0) || ((logfile_fd = open_crashlog()) >= 0))
+    DUMP(logfile_fd)
   if (!zlog_default)
-    DUMP(stderr)
+    DUMP(STDERR_FILENO)
   else
     {
-      if ((PRI <= zlog_default->maxlvl[ZLOG_DEST_FILE]) && zlog_default->fp)
-        DUMP(zlog_default->fp)
       if (PRI <= zlog_default->maxlvl[ZLOG_DEST_STDOUT])
-        DUMP(stdout)
+        DUMP(STDOUT_FILENO)
       /* Remove trailing '\n' for monitor and syslog */
       *--s = '\0';
       if (PRI <= zlog_default->maxlvl[ZLOG_DEST_MONITOR])
@@ -337,7 +371,7 @@ zlog_signal(int signo, const char *action
 void
 zlog_backtrace_sigsafe(int priority, void *program_counter)
 {
-#ifdef HAVE_GLIBC_BACKTRACE
+#ifdef HAVE_STACK_TRACE
   static const char pclabel[] = "Program counter: ";
   void *array[20];
   int size;
@@ -345,33 +379,42 @@ zlog_backtrace_sigsafe(int priority, void *program_counter)
   char *s;
 #define LOC s,buf+sizeof(buf)-s
 
+#ifdef HAVE_GLIBC_BACKTRACE
   if (((size = backtrace(array,sizeof(array)/sizeof(array[0]))) <= 0) ||
       ((size_t)size > sizeof(array)/sizeof(array[0])))
     return;
+
+#define DUMP(FD) { \
+  if (program_counter) \
+    { \
+      write(FD, pclabel, sizeof(pclabel)-1); \
+      backtrace_symbols_fd(&program_counter, 1, FD); \
+    } \
+  write(FD, buf, s-buf);	\
+  backtrace_symbols_fd(array, size, FD); \
+}
+#elif defined(HAVE_PRINTSTACK)
+#define DUMP(FD) { \
+  if (program_counter) \
+    write((FD), pclabel, sizeof(pclabel)-1); \
+  write((FD), buf, s-buf); \
+  printstack((FD)); \
+}
+#endif /* HAVE_GLIBC_BACKTRACE, HAVE_PRINTSTACK */
+
   s = buf;
   s = str_append(LOC,"Backtrace for ");
   s = num_append(LOC,size);
   s = str_append(LOC," stack frames:\n");
 
-#define DUMP(FP) { \
-  if (program_counter) \
-    { \
-      write(fileno(FP),pclabel,sizeof(pclabel)-1); \
-      backtrace_symbols_fd(&program_counter, 1, fileno(FP)); \
-    } \
-  write(fileno(FP),buf,s-buf);	\
-  backtrace_symbols_fd(array, size, fileno(FP)); \
-}
-
+  if ((logfile_fd >= 0) || ((logfile_fd = open_crashlog()) >= 0))
+    DUMP(logfile_fd)
   if (!zlog_default)
-    DUMP(stderr)
+    DUMP(STDERR_FILENO)
   else
     {
-      if ((priority <= zlog_default->maxlvl[ZLOG_DEST_FILE]) &&
-	  zlog_default->fp)
-	DUMP(zlog_default->fp)
       if (priority <= zlog_default->maxlvl[ZLOG_DEST_STDOUT])
-	DUMP(stdout)
+	DUMP(STDOUT_FILENO)
       /* Remove trailing '\n' for monitor and syslog */
       *--s = '\0';
       if (priority <= zlog_default->maxlvl[ZLOG_DEST_MONITOR])
@@ -398,7 +441,7 @@ zlog_backtrace_sigsafe(int priority, void *program_counter)
     }
 #undef DUMP
 #undef LOC
-#endif /* HAVE_GLIBC_BACKTRACE */
+#endif /* HAVE_STRACK_TRACE */
 }
 
 void
@@ -493,9 +536,14 @@ void
 _zlog_assert_failed (const char *assertion, const char *file,
 		     unsigned int line, const char *function)
 {
-  zlog_err("Assertion `%s' failed in file %s, line %u, function %s",
-	   assertion,file,line,(function ? function : "?"));
-  zlog_backtrace(LOG_ERR);
+  /* Force fallback file logging? */
+  if (zlog_default && !zlog_default->fp &&
+      ((logfile_fd = open_crashlog()) >= 0) &&
+      ((zlog_default->fp = fdopen(logfile_fd, "w")) != NULL))
+    zlog_default->maxlvl[ZLOG_DEST_FILE] = LOG_ERR;
+  zlog(NULL, LOG_CRIT, "Assertion `%s' failed in file %s, line %u, function %s",
+       assertion,file,line,(function ? function : "?"));
+  zlog_backtrace(LOG_CRIT);
   abort();
 }
 
@@ -569,6 +617,7 @@ zlog_set_file (struct zlog *zl, const char *filename, int log_level)
   zl->filename = strdup (filename);
   zl->maxlvl[ZLOG_DEST_FILE] = log_level;
   zl->fp = fp;
+  logfile_fd = fileno(fp);
 
   return 1;
 }
@@ -583,6 +632,7 @@ zlog_reset_file (struct zlog *zl)
   if (zl->fp)
     fclose (zl->fp);
   zl->fp = NULL;
+  logfile_fd = -1;
   zl->maxlvl[ZLOG_DEST_FILE] = ZLOG_DISABLED;
 
   if (zl->filename)
@@ -604,6 +654,7 @@ zlog_rotate (struct zlog *zl)
   if (zl->fp)
     fclose (zl->fp);
   zl->fp = NULL;
+  logfile_fd = -1;
   level = zl->maxlvl[ZLOG_DEST_FILE];
   zl->maxlvl[ZLOG_DEST_FILE] = ZLOG_DISABLED;
 
@@ -622,6 +673,7 @@ zlog_rotate (struct zlog *zl)
 	  	   zl->filename, safe_strerror(save_errno));
 	  return -1;
         }	
+      logfile_fd = fileno(zl->fp);
       zl->maxlvl[ZLOG_DEST_FILE] = level;
     }
 
@@ -661,3 +713,103 @@ safe_strerror(int errnum)
   const char *s = strerror(errnum);
   return (s != NULL) ? s : "Unknown error";
 }
+
+struct zebra_desc_table
+{
+  unsigned int type;
+  const char *string;
+  char chr;
+};
+
+#define DESC_ENTRY(T,S,C) [(T)] = { (T), (S), (C) }
+static const struct zebra_desc_table route_types[] = {
+  DESC_ENTRY	(ZEBRA_ROUTE_SYSTEM,	"system",	'X' ),
+  DESC_ENTRY	(ZEBRA_ROUTE_KERNEL,	"kernel",	'K' ),
+  DESC_ENTRY	(ZEBRA_ROUTE_CONNECT,	"connected",	'C' ),
+  DESC_ENTRY	(ZEBRA_ROUTE_STATIC,	"static",	'S' ),
+  DESC_ENTRY	(ZEBRA_ROUTE_RIP,	"rip",		'R' ),
+  DESC_ENTRY	(ZEBRA_ROUTE_RIPNG,	"ripng",	'R' ),
+  DESC_ENTRY	(ZEBRA_ROUTE_OSPF,	"ospf",		'O' ),
+  DESC_ENTRY	(ZEBRA_ROUTE_OSPF6,	"ospf6",	'O' ),
+  DESC_ENTRY	(ZEBRA_ROUTE_ISIS,	"isis",		'I' ),
+  DESC_ENTRY	(ZEBRA_ROUTE_BGP,	"bgp",		'B' ),
+  DESC_ENTRY	(ZEBRA_ROUTE_HSLS,	"hsls",		'H' ),
+};
+#undef DESC_ENTRY
+
+#define DESC_ENTRY(T) [(T)] = { (T), (#T), '\0' }
+static const struct zebra_desc_table command_types[] = {
+  DESC_ENTRY	(ZEBRA_INTERFACE_ADD),
+  DESC_ENTRY	(ZEBRA_INTERFACE_DELETE),
+  DESC_ENTRY	(ZEBRA_INTERFACE_ADDRESS_ADD),
+  DESC_ENTRY	(ZEBRA_INTERFACE_ADDRESS_DELETE),
+  DESC_ENTRY	(ZEBRA_INTERFACE_UP),
+  DESC_ENTRY	(ZEBRA_INTERFACE_DOWN),
+  DESC_ENTRY	(ZEBRA_IPV4_ROUTE_ADD),
+  DESC_ENTRY	(ZEBRA_IPV4_ROUTE_DELETE),
+  DESC_ENTRY	(ZEBRA_IPV6_ROUTE_ADD),
+  DESC_ENTRY	(ZEBRA_IPV6_ROUTE_DELETE),
+  DESC_ENTRY	(ZEBRA_REDISTRIBUTE_ADD),
+  DESC_ENTRY	(ZEBRA_REDISTRIBUTE_DELETE),
+  DESC_ENTRY	(ZEBRA_REDISTRIBUTE_DEFAULT_ADD),
+  DESC_ENTRY	(ZEBRA_REDISTRIBUTE_DEFAULT_DELETE),
+  DESC_ENTRY	(ZEBRA_IPV4_NEXTHOP_LOOKUP),
+  DESC_ENTRY	(ZEBRA_IPV6_NEXTHOP_LOOKUP),
+  DESC_ENTRY	(ZEBRA_IPV4_IMPORT_LOOKUP),
+  DESC_ENTRY	(ZEBRA_IPV6_IMPORT_LOOKUP),
+  DESC_ENTRY	(ZEBRA_INTERFACE_RENAME),
+  DESC_ENTRY	(ZEBRA_ROUTER_ID_ADD),
+  DESC_ENTRY	(ZEBRA_ROUTER_ID_DELETE),
+  DESC_ENTRY	(ZEBRA_ROUTER_ID_UPDATE),
+};
+#undef DESC_ENTRY
+
+static const struct zebra_desc_table unknown = { 0, "unknown", '?' };
+
+static const struct zebra_desc_table *
+zroute_lookup(u_int zroute)
+{
+  u_int i;
+
+  if (zroute >= sizeof(route_types)/sizeof(route_types[0]))
+    {
+      zlog_err("unknown zebra route type: %u", zroute);
+      return &unknown;
+    }
+  if (zroute == route_types[zroute].type)
+    return &route_types[zroute];
+  for (i = 0; i < sizeof(route_types)/sizeof(route_types[0]); i++)
+    {
+      if (zroute == route_types[i].type)
+        {
+	  zlog_warn("internal error: route type table out of order "
+		    "while searching for %u, please notify developers", zroute);
+	  return &route_types[i];
+        }
+    }
+  zlog_err("internal error: cannot find route type %u in table!", zroute);
+  return &unknown;
+}
+
+const char *
+zebra_route_string(u_int zroute)
+{
+  return zroute_lookup(zroute)->string;
+}
+
+char
+zebra_route_char(u_int zroute)
+{
+  return zroute_lookup(zroute)->chr;
+}
+
+const char *
+zserv_command_string (unsigned int command)
+{
+  if (command >= sizeof(command_types)/sizeof(command_types[0]))
+    {
+      zlog_err ("unknown zserv command type: %u", command);
+      return unknown.string;
+    }
+  return command_types[command].string;
+}
