@@ -6,7 +6,7 @@
  * The script is sensitive to the format (though not whitespace), see
  * the top of memtypes.awk for more details.
  *
- * $Id: memtypes.c,v 1.4 2005/04/25 16:26:43 paul Exp $
+ * $Id: memtypes.c,v 1.11 2006/03/30 14:09:38 paul Exp $
  */
 
 #include "zebra.h"
@@ -29,6 +29,7 @@ struct memory_list memory_list_lib[] =
   { MTYPE_VTY_HIST,		"VTY history"			},
   { MTYPE_IF,			"Interface"			},
   { MTYPE_CONNECTED,		"Connected" 			},
+  { MTYPE_CONNECTED_LABEL,	"Connected interface label"	},
   { MTYPE_BUFFER,		"Buffer"			},
   { MTYPE_BUFFER_DATA,		"Buffer data"			},
   { MTYPE_STREAM,		"Stream"			},
@@ -60,6 +61,7 @@ struct memory_list memory_list_lib[] =
   { MTYPE_KEY,			"Key"				},
   { MTYPE_KEYCHAIN,		"Key chain"			},
   { MTYPE_IF_RMAP,		"Interface route map"		},
+  { MTYPE_IF_RMAP_NAME,		"I.f. route map name",		},
   { MTYPE_SOCKUNION,		"Socket union"			},
   { MTYPE_PRIVS,		"Privilege information"		},
   { MTYPE_ZLOG,			"Logging"			},
@@ -67,6 +69,9 @@ struct memory_list memory_list_lib[] =
   { MTYPE_WORK_QUEUE,		"Work queue"			},
   { MTYPE_WORK_QUEUE_ITEM,	"Work queue item"		},
   { MTYPE_WORK_QUEUE_NAME,	"Work queue name string"	},
+  { MTYPE_PQUEUE,		"Priority queue"		},
+  { MTYPE_PQUEUE_DATA,		"Priority queue data"		},
+  { MTYPE_HOST,			"Host config"			},
   { -1, NULL },
 };
 
@@ -87,11 +92,13 @@ struct memory_list memory_list_bgp[] =
 {
   { MTYPE_BGP,			"BGP instance"			},
   { MTYPE_BGP_PEER,		"BGP peer"			},
+  { MTYPE_BGP_PEER_HOST,	"BGP peer hostname"		},
   { MTYPE_PEER_GROUP,		"Peer group"			},
   { MTYPE_PEER_DESC,		"Peer description"		},
   { MTYPE_ATTR,			"BGP attribute"			},
   { MTYPE_AS_PATH,		"BGP aspath"			},
   { MTYPE_AS_SEG,		"BGP aspath seg"		},
+  { MTYPE_AS_SEG_DATA,		"BGP aspath segment data"	},
   { MTYPE_AS_STR,		"BGP aspath str"		},
   { 0, NULL },
   { MTYPE_BGP_TABLE,		"BGP table"			},
@@ -100,6 +107,7 @@ struct memory_list memory_list_bgp[] =
   { MTYPE_BGP_STATIC,		"BGP static"			},
   { MTYPE_BGP_ADVERTISE_ATTR,	"BGP adv attr"			},
   { MTYPE_BGP_ADVERTISE,	"BGP adv"			},
+  { MTYPE_BGP_SYNCHRONISE,	"BGP synchronise"		},
   { MTYPE_BGP_ADJ_IN,		"BGP adj in"			},
   { MTYPE_BGP_ADJ_OUT,		"BGP adj out"			},
   { 0, NULL },
@@ -124,6 +132,9 @@ struct memory_list memory_list_bgp[] =
   { MTYPE_CLUSTER,		"Cluster list"			},
   { MTYPE_CLUSTER_VAL,		"Cluster list val"		},
   { 0, NULL },
+  { MTYPE_BGP_PROCESS_QUEUE,	"BGP Process queue"		},
+  { MTYPE_BGP_CLEAR_NODE_QUEUE, "BGP node clear queue"		},
+  { 0, NULL },
   { MTYPE_TRANSIT,		"BGP transit attr"		},
   { MTYPE_TRANSIT_VAL,		"BGP transit val"		},
   { 0, NULL },
@@ -177,6 +188,7 @@ struct memory_list memory_list_ospf[] =
   { MTYPE_OSPF_PACKET,        "OSPF packet"			},
   { MTYPE_OSPF_FIFO,          "OSPF FIFO queue"			},
   { MTYPE_OSPF_VERTEX,        "OSPF vertex"			},
+  { MTYPE_OSPF_VERTEX_PARENT, "OSPF vertex parent",		},
   { MTYPE_OSPF_NEXTHOP,       "OSPF nexthop"			},
   { MTYPE_OSPF_PATH,	      "OSPF path"			},
   { MTYPE_OSPF_VL_DATA,       "OSPF VL data"			},
