@@ -26,6 +26,7 @@ Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 #include "prefix.h"
 #include "routemap.h"
 #include "command.h"
+#include "vty.h"
 #include "log.h"
 
 /* Vector for route match rules. */
@@ -159,7 +160,7 @@ route_map_lookup_by_name (const char *name)
 
 /* Lookup route map.  If there isn't route map create one and return
    it. */
-struct route_map *
+static struct route_map *
 route_map_get (const char *name)
 {
   struct route_map *map;
@@ -171,7 +172,7 @@ route_map_get (const char *name)
 }
 
 /* Return route map's type string. */
-const static char *
+static const char *
 route_map_type_str (enum route_map_type type)
 {
   switch (type)
@@ -188,7 +189,7 @@ route_map_type_str (enum route_map_type type)
     }
 }
 
-int
+static int
 route_map_empty (struct route_map *map)
 {
   if (map->head == NULL && map->tail == NULL)
@@ -204,6 +205,11 @@ vty_show_route_map_entry (struct vty *vty, struct route_map *map)
   struct route_map_index *index;
   struct route_map_rule *rule;
 
+  /* Print the name of the protocol */
+  if (zlog_default)
+    vty_out (vty, "%s:%s", zlog_proto_names[zlog_default->protocol],
+             VTY_NEWLINE);
+
   for (index = map->head; index; index = index->next)
     {
       vty_out (vty, "route-map %s, %s, sequence %d%s",
@@ -226,26 +232,23 @@ vty_show_route_map_entry (struct vty *vty, struct route_map *map)
         vty_out (vty, "    %s %s%s",
                  rule->cmd->str, rule->rule_str, VTY_NEWLINE);
       
-      vty_out (vty, "  Action:%s", VTY_NEWLINE);
-      
+      /* Call clause */
+      vty_out (vty, "  Call clause:%s", VTY_NEWLINE);
       if (index->nextrm)
         vty_out (vty, "    Call %s%s", index->nextrm, VTY_NEWLINE);
-      else if (index->exitpolicy == RMAP_GOTO)
+      
+      /* Exit Policy */
+      vty_out (vty, "  Action:%s", VTY_NEWLINE);
+      if (index->exitpolicy == RMAP_GOTO)
         vty_out (vty, "    Goto %d%s", index->nextpref, VTY_NEWLINE);
       else if (index->exitpolicy == RMAP_NEXT)
-        {
-          vty_out (vty, "    Goto next, (entry ");
-          if (index->next)
-            vty_out (vty, " %d)%s", index->next->pref, VTY_NEWLINE);
-          else
-            vty_out (vty, " undefined)%s", VTY_NEWLINE);
-        }
+        vty_out (vty, "    Continue to next entry%s", VTY_NEWLINE);
       else if (index->exitpolicy == RMAP_EXIT)
         vty_out (vty, "    Exit routemap%s", VTY_NEWLINE);
     }
 }
 
-int
+static int
 vty_show_route_map (struct vty *vty, const char *name)
 {
   struct route_map *map;
@@ -271,8 +274,8 @@ vty_show_route_map (struct vty *vty, const char *name)
 
 /* New route map allocation. Please note route map's name must be
    specified. */
-struct route_map_index *
-route_map_index_new ()
+static struct route_map_index *
+route_map_index_new (void)
 {
   struct route_map_index *new;
 
@@ -308,7 +311,7 @@ route_map_index_delete (struct route_map_index *index, int notify)
 
   /* Free 'char *nextrm' if not NULL */
   if (index->nextrm)
-    free (index->nextrm);
+    XFREE (MTYPE_ROUTE_MAP_NAME, index->nextrm);
 
     /* Execute event hook. */
   if (route_map_master.event_hook && notify)
@@ -319,7 +322,7 @@ route_map_index_delete (struct route_map_index *index, int notify)
 }
 
 /* Lookup index from route map. */
-struct route_map_index *
+static struct route_map_index *
 route_map_index_lookup (struct route_map *map, enum route_map_type type,
 			int pref)
 {
@@ -333,7 +336,7 @@ route_map_index_lookup (struct route_map *map, enum route_map_type type,
 }
 
 /* Add new index to route map. */
-struct route_map_index *
+static struct route_map_index *
 route_map_index_add (struct route_map *map, enum route_map_type type,
 		     int pref)
 {
@@ -385,7 +388,7 @@ route_map_index_add (struct route_map *map, enum route_map_type type,
 }
 
 /* Get route map index. */
-struct route_map_index *
+static struct route_map_index *
 route_map_index_get (struct route_map *map, enum route_map_type type, 
 		     int pref)
 {
@@ -404,8 +407,8 @@ route_map_index_get (struct route_map *map, enum route_map_type type,
 }
 
 /* New route map rule */
-struct route_map_rule *
-route_map_rule_new ()
+static struct route_map_rule *
+route_map_rule_new (void)
 {
   struct route_map_rule *new;
 
@@ -428,7 +431,7 @@ route_map_install_set (struct route_map_rule_cmd *cmd)
 }
 
 /* Lookup rule command from match list. */
-struct route_map_rule_cmd *
+static struct route_map_rule_cmd *
 route_map_lookup_match (const char *name)
 {
   unsigned int i;
@@ -442,7 +445,7 @@ route_map_lookup_match (const char *name)
 }
 
 /* Lookup rule command from set list. */
-struct route_map_rule_cmd *
+static struct route_map_rule_cmd *
 route_map_lookup_set (const char *name)
 {
   unsigned int i;
@@ -493,7 +496,7 @@ route_map_rule_delete (struct route_map_rule_list *list,
 }
 
 /* strcmp wrapper function which don't crush even argument is NULL. */
-int
+static int
 rulecmp (const char *dst, const char *src)
 {
   if (dst == NULL)
@@ -731,7 +734,7 @@ route_map_delete_set (struct route_map_index *index, const char *set_name,
    We need to make sure our route-map processing matches the above
 */
 
-route_map_result_t
+static route_map_result_t
 route_map_apply_match (struct route_map_rule_list *match_list,
                        struct prefix *prefix, route_map_object_t type,
                        void *object)
@@ -875,7 +878,7 @@ route_map_event_hook (void (*func) (route_map_event_t, const char *))
 }
 
 void
-route_map_init ()
+route_map_init (void)
 {
   /* Make vector for match and set. */
   route_match_vec = vector_init (1);
@@ -1063,15 +1066,16 @@ DEFUN (rmap_onmatch_goto,
        "Goto Clause number\n"
        "Number\n")
 {
-  struct route_map_index *index;
+  struct route_map_index *index = vty->index;
   int d = 0;
 
-  if (argv[0])
-    d = atoi(argv[0]);
-
-  index = vty->index;
   if (index)
     {
+      if (argc == 1 && argv[0])
+        VTY_GET_INTEGER_RANGE("route-map index", d, argv[0], 1, 65536);
+      else
+        d = index->pref + 1;
+      
       if (d <= index->pref)
 	{
 	  /* Can't allow you to do that, Dave */
@@ -1168,8 +1172,8 @@ DEFUN (rmap_call,
   if (index)
     {
       if (index->nextrm)
-          free (index->nextrm);
-      index->nextrm = strdup (argv[0]);
+          XFREE (MTYPE_ROUTE_MAP_NAME, index->nextrm);
+      index->nextrm = XSTRDUP (MTYPE_ROUTE_MAP_NAME, argv[0]);
     }
   return CMD_SUCCESS;
 }
@@ -1186,7 +1190,7 @@ DEFUN (no_rmap_call,
 
   if (index->nextrm)
     {
-      free (index->nextrm);
+      XFREE (MTYPE_ROUTE_MAP_NAME, index->nextrm);
       index->nextrm = NULL;
     }
 
@@ -1230,7 +1234,7 @@ DEFUN (no_rmap_description,
 }
 
 /* Configuration write function. */
-int
+static int
 route_map_config_write (struct vty *vty)
 {
   struct route_map *map;
@@ -1286,7 +1290,7 @@ struct cmd_node rmap_node =
 
 /* Initialization of route map vector. */
 void
-route_map_init_vty ()
+route_map_init_vty (void)
 {
   /* Install route map top node. */
   install_node (&rmap_node, route_map_config_write);
