@@ -46,7 +46,7 @@
 /* ospf6d privileges */
 zebra_capabilities_t _caps_p [] =
 {
-  ZCAP_RAW,
+  ZCAP_NET_RAW,
   ZCAP_BIND
 };
 
