@@ -1119,10 +1119,9 @@ ospf6_lsack_recv (struct in6_addr *src, struct in6_addr *dst,
         zlog_debug ("Acknowledged, remove from %s's retrans-list",
 		    on->name);
 
+      ospf6_decrement_retrans_count (mine);
       if (OSPF6_LSA_IS_MAXAGE (mine))
         ospf6_maxage_remove (on->ospf6_if->area->ospf6);
-
-      ospf6_decrement_retrans_count (mine);
       ospf6_lsdb_remove (mine, on->retrans_list);
       ospf6_lsa_delete (his);
     }
@@ -1186,6 +1185,9 @@ ospf6_receive (struct thread *thread)
   thread_add_read (master, ospf6_receive, NULL, sockfd);
 
   /* initialize */
+  memset (&src, 0, sizeof (src));
+  memset (&dst, 0, sizeof (dst));
+  ifindex = 0;
   memset (recvbuf, 0, iobuflen);
   iovector[0].iov_base = recvbuf;
   iovector[0].iov_len = iobuflen;
