@@ -29,27 +29,145 @@
 #include "rib.h"
 #include "table.h"
 #include "log.h"
+#include "memory.h"
 
 #include "zebra/zserv.h"
 #include "zebra/redistribute.h"
+#include "zebra/interface.h"
+#include "zebra/connected.h"
+
+/* withdraw a connected address */
+static void
+connected_withdraw (struct connected *ifc)
+{
+  if (! ifc)
+    return;
+
+  /* Update interface address information to protocol daemon. */
+  if (CHECK_FLAG (ifc->conf, ZEBRA_IFC_REAL))
+    {
+      zebra_interface_address_delete_update (ifc->ifp, ifc);
+
+      if_subnet_delete (ifc->ifp, ifc);
+      
+      if (ifc->address->family == AF_INET)
+        connected_down_ipv4 (ifc->ifp, ifc);
+#ifdef HAVE_IPV6
+      else
+        connected_down_ipv6 (ifc->ifp, ifc);
+#endif
+
+      UNSET_FLAG (ifc->conf, ZEBRA_IFC_REAL);
+    }
+
+  if (!CHECK_FLAG (ifc->conf, ZEBRA_IFC_CONFIGURED))
+    {
+      listnode_delete (ifc->ifp->connected, ifc);
+      connected_free (ifc);
+    }
+}
+
+static void
+connected_announce (struct interface *ifp, struct connected *ifc)
+{
+  if (!ifc)
+    return;
+  
+  listnode_add (ifp->connected, ifc);
+
+  /* Update interface address information to protocol daemon. */
+  if (! CHECK_FLAG (ifc->conf, ZEBRA_IFC_REAL))
+    {
+      if (ifc->address->family == AF_INET)
+        if_subnet_add (ifp, ifc);
+
+      SET_FLAG (ifc->conf, ZEBRA_IFC_REAL);
+
+      zebra_interface_address_add_update (ifp, ifc);
+
+      if (if_is_up(ifp))
+        {
+          if (ifc->address->family == AF_INET)
+	    connected_up_ipv4 (ifp, ifc);
+#ifdef HAVE_IPV6
+          else
+            connected_up_ipv6 (ifp, ifc);
+#endif
+        }
+    }
+}
 
 /* If same interface address is already exist... */
 struct connected *
-connected_check_ipv4 (struct interface *ifp, struct prefix *p)
+connected_check (struct interface *ifp, struct prefix *p)
 {
   struct connected *ifc;
   struct listnode *node;
 
-  for (node = listhead (ifp->connected); node; node = nextnode (node))
-    {
-      ifc = getdata (node);
+  for (ALL_LIST_ELEMENTS_RO (ifp->connected, node, ifc))
+    if (prefix_same (ifc->address, p))
+      return ifc;
 
-      if (prefix_same (ifc->address, p))
-	return ifc;
-    }
   return NULL;
 }
 
+/* Check if two ifc's describe the same address */
+static int
+connected_same (struct connected *ifc1, struct connected *ifc2)
+{
+  if (ifc1->ifp != ifc2->ifp)
+    return 0;
+  
+  if (ifc1->destination)
+    if (!ifc2->destination)
+      return 0;
+  if (ifc2->destination)
+    if (!ifc1->destination)
+      return 0;
+  
+  if (ifc1->destination && ifc2->destination)
+    if (!prefix_same (ifc1->destination, ifc2->destination))
+      return 0;
+
+  if (ifc1->flags != ifc2->flags)
+    return 0;
+  
+  return 1;
+}
+
+/* Handle implicit withdrawals of addresses, where a system ADDs an address
+ * to an interface which already has the same address configured.
+ *
+ * Returns the struct connected which must be announced to clients,
+ * or NULL if nothing to do.
+ */
+static struct connected *
+connected_implicit_withdraw (struct interface *ifp, struct connected *ifc)
+{
+  struct connected *current;
+  
+  /* Check same connected route. */
+  if ((current = connected_check (ifp, (struct prefix *) ifc->address)))
+    {
+      if (CHECK_FLAG(current->conf, ZEBRA_IFC_CONFIGURED))
+        SET_FLAG(ifc->conf, ZEBRA_IFC_CONFIGURED);
+	
+      /* Avoid spurious withdraws, this might be just the kernel 'reflecting'
+       * back an address we have already added.
+       */
+      if (connected_same (current, ifc))
+        {
+          /* nothing to do */
+          connected_free (ifc);
+          return NULL;
+        }
+      
+      UNSET_FLAG(current->conf, ZEBRA_IFC_CONFIGURED);
+      connected_withdraw (current); /* implicit withdraw - freebsd does this */
+    }
+  return ifc;
+}
+
 /* Called from if_up(). */
 void
 connected_up_ipv4 (struct interface *ifp, struct connected *ifc)
@@ -82,7 +200,8 @@ connected_up_ipv4 (struct interface *ifp, struct connected *ifc)
   if (prefix_ipv4_any (&p))
     return;
 
-  rib_add_ipv4 (ZEBRA_ROUTE_CONNECT, 0, &p, NULL, ifp->ifindex, 0, 0, 0);
+  rib_add_ipv4 (ZEBRA_ROUTE_CONNECT, 0, &p, NULL, ifp->ifindex, RT_TABLE_MAIN, 
+                ifp->metric, 0);
 
   rib_update ();
 }
@@ -90,11 +209,11 @@ connected_up_ipv4 (struct interface *ifp, struct connected *ifc)
 /* Add connected IPv4 route to the interface. */
 void
 connected_add_ipv4 (struct interface *ifp, int flags, struct in_addr *addr, 
-		    int prefixlen, struct in_addr *broad, char *label)
+		    u_char prefixlen, struct in_addr *broad, 
+		    const char *label)
 {
   struct prefix_ipv4 *p;
   struct connected *ifc;
-  struct connected *current;
 
   /* Make connected structure. */
   ifc = connected_new ();
@@ -107,7 +226,7 @@ connected_add_ipv4 (struct interface *ifp, int flags, struct in_addr *addr,
   p->prefix = *addr;
   p->prefixlen = prefixlen;
   ifc->address = (struct prefix *) p;
-
+  
   /* If there is broadcast or pointopoint address. */
   if (broad)
     {
@@ -161,30 +280,13 @@ connected_add_ipv4 (struct interface *ifp, int flags, struct in_addr *addr,
 
   /* Label of this address. */
   if (label)
-    ifc->label = strdup (label);
+    ifc->label = XSTRDUP (MTYPE_CONNECTED_LABEL, label);
 
-  /* Check same connected route. */
-  current = connected_check_ipv4 (ifp, (struct prefix *) ifc->address);
-  if (current)
-    {
-      connected_free (ifc);
-      ifc = current;
-    }
-  else
-    {
-      listnode_add (ifp->connected, ifc);
-    }
-
-  /* Update interface address information to protocol daemon. */
-  if (! CHECK_FLAG (ifc->conf, ZEBRA_IFC_REAL))
-    {
-      SET_FLAG (ifc->conf, ZEBRA_IFC_REAL);
-
-      zebra_interface_address_add_update (ifp, ifc);
-
-      if (if_is_up(ifp))
-	connected_up_ipv4 (ifp, ifc);
-    }
+  /* nothing to do? */
+  if ((ifc = connected_implicit_withdraw (ifp, ifc)) == NULL)
+    return;
+  
+  connected_announce (ifp, ifc);
 }
 
 void
@@ -226,7 +328,7 @@ connected_down_ipv4 (struct interface *ifp, struct connected *ifc)
 /* Delete connected IPv4 route to the interface. */
 void
 connected_delete_ipv4 (struct interface *ifp, int flags, struct in_addr *addr,
-		       int prefixlen, struct in_addr *broad, char *label)
+		       u_char prefixlen, struct in_addr *broad)
 {
   struct prefix_ipv4 p;
   struct connected *ifc;
@@ -236,45 +338,14 @@ connected_delete_ipv4 (struct interface *ifp, int flags, struct in_addr *addr,
   p.prefix = *addr;
   p.prefixlen = prefixlen;
 
-  ifc = connected_check_ipv4 (ifp, (struct prefix *) &p);
+  ifc = connected_check (ifp, (struct prefix *) &p);
   if (! ifc)
     return;
-
-  /* Update interface address information to protocol daemon. */
-  if (CHECK_FLAG (ifc->conf, ZEBRA_IFC_REAL))
-    {
-      zebra_interface_address_delete_update (ifp, ifc);
-
-      connected_down_ipv4 (ifp, ifc);
-
-      UNSET_FLAG (ifc->conf, ZEBRA_IFC_REAL);
-    }
-
-  if (! CHECK_FLAG (ifc->conf, ZEBRA_IFC_CONFIGURED))
-    {
-      listnode_delete (ifp->connected, ifc);
-      connected_free (ifc);
-    }
+    
+  connected_withdraw (ifc);
 }
 
 #ifdef HAVE_IPV6
-/* If same interface address is already exist... */
-struct connected *
-connected_check_ipv6 (struct interface *ifp, struct prefix *p)
-{
-  struct connected *ifc;
-  struct listnode *node;
-
-  for (node = listhead (ifp->connected); node; node = nextnode (node))
-    {
-      ifc = getdata (node);
-
-      if (prefix_same (ifc->address, p))
-	return ifc;
-    }
-  return 0;
-}
-
 void
 connected_up_ipv6 (struct interface *ifp, struct connected *ifc)
 {
@@ -311,7 +382,8 @@ connected_up_ipv6 (struct interface *ifp, struct connected *ifc)
     return;
 #endif
 
-  rib_add_ipv6 (ZEBRA_ROUTE_CONNECT, 0, &p, NULL, ifp->ifindex, 0);
+  rib_add_ipv6 (ZEBRA_ROUTE_CONNECT, 0, &p, NULL, ifp->ifindex, 0,
+                ifp->metric, 0);
 
   rib_update ();
 }
@@ -319,11 +391,11 @@ connected_up_ipv6 (struct interface *ifp, struct connected *ifc)
 /* Add connected IPv6 route to the interface. */
 void
 connected_add_ipv6 (struct interface *ifp, struct in6_addr *addr,
-		    u_char prefixlen, struct in6_addr *broad)
+		    u_char prefixlen, struct in6_addr *broad,
+		    const char *label)
 {
   struct prefix_ipv6 *p;
   struct connected *ifc;
-  struct connected *current;
 
   /* Make connected structure. */
   ifc = connected_new ();
@@ -345,27 +417,14 @@ connected_add_ipv6 (struct interface *ifp, struct in6_addr *addr,
       ifc->destination = (struct prefix *) p;
     }
 
-  current = connected_check_ipv6 (ifp, (struct prefix *) ifc->address);
-  if (current)
-    {
-      connected_free (ifc);
-      ifc = current;
-    }
-  else
-    {
-      listnode_add (ifp->connected, ifc);
-    }
-
-  /* Update interface address information to protocol daemon. */
-  if (! CHECK_FLAG (ifc->conf, ZEBRA_IFC_REAL))
-    {
-      SET_FLAG (ifc->conf, ZEBRA_IFC_REAL);
-
-      zebra_interface_address_add_update (ifp, ifc);
-
-      if (if_is_up(ifp))
-	connected_up_ipv6 (ifp, ifc);
-    }
+  /* Label of this address. */
+  if (label)
+    ifc->label = XSTRDUP (MTYPE_CONNECTED_LABEL, label);
+  
+  if ((ifc = connected_implicit_withdraw (ifp, ifc)) == NULL)
+    return;
+  
+  connected_announce (ifp, ifc);
 }
 
 void
@@ -417,24 +476,10 @@ connected_delete_ipv6 (struct interface *ifp, struct in6_addr *address,
   memcpy (&p.prefix, address, sizeof (struct in6_addr));
   p.prefixlen = prefixlen;
 
-  ifc = connected_check_ipv6 (ifp, (struct prefix *) &p);
+  ifc = connected_check (ifp, (struct prefix *) &p);
   if (! ifc)
     return;
 
-  /* Update interface address information to protocol daemon. */
-  if (CHECK_FLAG (ifc->conf, ZEBRA_IFC_REAL))
-    {
-      zebra_interface_address_delete_update (ifp, ifc);
-
-      connected_down_ipv6 (ifp, ifc);
-
-      UNSET_FLAG (ifc->conf, ZEBRA_IFC_REAL);
-    }
-
-  if (! CHECK_FLAG (ifc->conf, ZEBRA_IFC_CONFIGURED))
-    {
-      listnode_delete (ifp->connected, ifc);
-      connected_free (ifc);
-    }
+  connected_withdraw (ifc);
 }
 #endif /* HAVE_IPV6 */
